
c     Subroutine on this file:
c      write_nc_drifter       - main subroutine for creating a netCDF
c                               dataset with information from drifter
c                               trajectories

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


c     - NOTES FOR "write_nc_drifter" -


c     **************
c     General notes:
c     **************

c     1. The path of the include file ('netdcf.inc') must be
c        included when compiling!; e.g.,
c          f77 -c -I/usr/local/netcdf-3.4/include write_nc_drifter.f

c     2. "write_nc_drifter" assumes that each time the subroutine is
c        entered, the netCDF file is in "data mode"
c        [except when it is not existing,
c         i.e., when 'openfile' is (.true.)].

c     3. "write_nc_drifter" returns in "data mode".

c     4. "write_nc_drifter" does not close the netCDF-file.


c     *********************************************
c     List of implemented parameters and variables:
c     *********************************************

c      (*1), (*2) : See comment below

c      parameter (short/long)          type   units        id
c      ^^^^^^^^^^^^^^^^^^^^^^          ^^^^   ^^^^^        ^^

c     COMPULSORY:
c      lon   / Longitude               float  degrees      lon_id
c      lat   / Latitude                float  degrees      lat_id
c      time  / Time                    float  (*)          time_id

c     ANY OF:
c      level / Vertical level          float  m           level_id
c      temp  / Temperature             float  deg. Kelvin  temp_id
c      salt  / Salinity                float  PSU          salt_id
c      sst   / Sea surface temperature float  deg. Kelvin  sst_id
c      sss   / Sea surface salinity    float  PSU          sss_id
c      sgm_t / Sigma-t (density)       float  kg/(m**3)    sgm_t_id
c      u     / Velocity-x              float  m/s          u_id
c      v     / Velocity-y              float  m/s          v_id
c      ssh   / Sea surface height      float  m            ssh_id

c     (*):
c     'time' is set relative to 'reference_date', with units as specified
c      by 'timeunit'. Recognized values of 'timeunit' are listed in the
c      section "Input/output variables" below.


c     ***********************
c     Input/output variables:
c     ***********************

c      openfile       (l)  - must be set to (.true.) when the netCDF file
c                             does not exist before "write_nc_drifter" is
c                             called for the first time;
c                            will be reset and output as (.false.) from
c                             "write_nc_drifter", and must not be changed
c                             thereafter
c      drifter_no     (i)  - the no. of the drifter that will be written
c                             to the netCDF file by the present call to
c                             "write_nc_drifter"
c      n_positions    (i)  - no. of positions where data is available from
c                             the present drifter ('drifter_no');
c                            only used when 'write_header' is (.true.)
c      write_header   (l)  - must be set to (.true.) when the first
c                             variable for the present drifter is written
c                             to the netCDF file;
c                            will be reset and output as (.false.) from
c                             "write_nc_drifter", and must be set back to
c                             (.true.) when a new drifter will be added
c      name           (s)  - short name of variable;
c                            must be listed in the file
c                             "name_drifter.inc";
c                            the variable name that will be used depends
c                             on 'drifter_no', e.g., when
c                              name='sst', drifter_no=123
c                             this name will be 'sss_000123'
c      variable       (rA) - array that holds the 'n_position' data
c                             values for the variable 'name', which will
c                             be written to the netCDF file by the present
c                             call to "write_nc_drifter"
c      var_1dim       (iA) - reference to the netCDF ID of the dimension
c                             ('n_positions') of the present drifter data;
c                            will be set when 'write_header' is (.true.),
c                             and must not be touched outside
c                             "write_nc_drifter"
c      reference_date (iA) - reference date for the present drifter,
c                             typically the release date;
c                            format:
c                             / year, month, day, hour, minute, second / ;
c                            used as a compulsory attribute for the
c                             variable 'time'
c      timeunit       (s)  - unit used for the 'time' variable;
c                            format: one of
c                             'years', 'months', 'days',
c                             'hours', 'minutes', 'seconds'

c     Above:
c      l  - logical varible  [ (.true.) / (.false.) ]
c      i  - integer variable [ 4 byte, aka. "long" ]
c      s  - string           [ character*(*) ]
c      iA - integer array    [ 4 byte, aka. "long" ]
c      rA - real array       [ 4 byte, aka. "float" ]


      subroutine write_nc_drifter(openfile,drifter_no,n_positions,
     >                           write_header,name,variable,var_1dim,
     >                           reference_date,timeunit)


      implicit none

      include 'netcdf.inc'
      include 'specs_drifter.inc'
      include 'id_drifter.inc'

c     Input:
      integer drifter_no,n_positions,reference_date(6)
      real variable(n_positions)
      character*(*) name
      character*(*) timeunit

c     Input/output:
      integer var_1dim(1)
      logical openfile,write_header

c     Local:
      integer maxdimension
      parameter (maxdimension= 999999)

      integer nc_status,size,long_size,filesize,time_size
      real range(2)
      character*19 date
      character*20 header_name,var_name
      character*50 var_long_name
      character*120 nc_string

c     Function:
      integer str_len
      logical ok_name_drifter


c     Check 'drifter_no':
      if (drifter_no .gt. maxdrifters .or. drifter_no .lt. 1) go to 997

c     Check 'maxdrifters':
      if (maxdrifters .gt. maxdimension) go to 998

c     Check 'name':
      if (.not. ok_name_drifter(name)) go to 999


      if (openfile) then

         filesize = str_len(filename)
         write(*,*)
         write(*,*)'======================================'
         write(*,*)' Attempting to create the netCDF file'
         write(*,*)'     ',filename(1:filesize)
         write(*,*)' for drifter data.'
         write(*,*)'======================================'

c        ************
c        netCDF Open:
c        ************

         nc_status = NF_CREATE(filename, NF_NOCLOBBER, nc_id)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 1)


c        *************************
c        netCDF global attributes:
c        *************************

c        General information:
c        --------------------

         size = str_len(title)
         nc_status = NF_PUT_ATT_TEXT(nc_id, NF_GLOBAL, 'title',
     >                               size, title(1:size))
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 11)

         size = str_len(source)
         nc_status = NF_PUT_ATT_TEXT(nc_id, NF_GLOBAL, 'source',
     >                               size, source(1:size))
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 12)

         size = str_len(method)
         nc_status = NF_PUT_ATT_TEXT(nc_id, NF_GLOBAL, 'method',
     >                               size, method(1:size))
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 13)

         size = str_len(reference)
         nc_status = NF_PUT_ATT_TEXT(nc_id, NF_GLOBAL, 'reference',
     >                               size, reference(1:size))
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 14)

         size = str_len(web_reference)
         nc_status = NF_PUT_ATT_TEXT(nc_id, NF_GLOBAL, 'web_reference',
     >                               size, web_reference(1:size))
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 15)

         size = str_len(history)
         nc_status = NF_PUT_ATT_TEXT(nc_id, NF_GLOBAL, 'history',
     >                              size,  history(1:size))
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 16)

         size = str_len(author)
         nc_status = NF_PUT_ATT_TEXT(nc_id, NF_GLOBAL, 'author',
     >                               size, author(1:size))
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 17)

         size = str_len(email)
         nc_status = NF_PUT_ATT_TEXT(nc_id, NF_GLOBAL, 'email',
     >                               size, email(1:size))
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 18)



c        ******************
c        netCDF dimensions:
c        ******************

         write(*,*)
         write(*,*)'Dimensions:'

         nc_status = NF_DEF_DIM(nc_id, 'drifter_no', NF_UNLIMITED,
     >                          no_id)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 19)
         write(*,*)' drifter_no:  NF_UNLIMITED'

c        Un-set 'openfile':
         openfile = .false.

      else

c        Put the netCDF file in "define mode":
         nc_status = NF_REDEF(nc_id)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 21)

      end if


c     Write header for this trajectory?

      if (write_header) then

         write(*,*)
         write(*,*)'Header:'

 101     format('n_positions_',i6.6)
         write(header_name,101)drifter_no
         size = str_len(header_name)

         nc_status = NF_DEF_DIM(nc_id, header_name, n_positions,
     >                          drifter_id(drifter_no))
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 31)
         write(*,*)header_name(1:size),': ',n_positions

         var_1dim(1) = drifter_id(drifter_no)

c        Un-set 'write_header':
         write_header = .false.

         write(*,*)
         write(*,*)'Variables:'

      end if


      size = str_len(name)

      if (name(1:size) .eq. 'lon') then

c        Longitude:

 201     format('lon_',i6.6)
         write(var_name,201)drifter_no
         size = str_len(var_name)
         nc_status = NF_DEF_VAR(nc_id, var_name(1:size), NF_FLOAT,
     >                          1, var_1dim, lon_id(drifter_no))
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 41)

 202     format('Longitude, drifter no. ',i6.6)
         write(var_long_name,202)drifter_no
         long_size = str_len(var_long_name)
         call nc_char_attrib(lon_id(drifter_no), 'long_name',
     >                       var_long_name(1:long_size),
     >                       var_name(1:size))
         call nc_char_attrib(lon_id(drifter_no), 'units',
     >                       'degrees_east', var_name(1:size))
         range(1) = -360.
         range(2) = 360.
         call nc_range(lon_id(drifter_no), range, var_name(1:size))

         nc_status = NF_ENDDEF(nc_id)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 42)
         nc_status = NF_PUT_VAR_REAL(nc_id, lon_id(drifter_no),
     >                               variable)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 43)

         write(*,*)drifter_no,
     >             ' Included variable   lon      / Longitude'


      else if (name(1:size) .eq. 'lat') then

c        Latitude:

 211     format('lat_',i6.6)
         write(var_name,211)drifter_no
         size = str_len(var_name)
         nc_status = NF_DEF_VAR(nc_id, var_name(1:size), NF_FLOAT,
     >                          1, var_1dim, lat_id(drifter_no))
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 51)

 212     format('Latitude, drifter no. ',i6.6)
         write(var_long_name,212)drifter_no
         long_size = str_len(var_long_name)
         call nc_char_attrib(lat_id(drifter_no), 'long_name',
     >                       var_long_name(1:long_size),
     >                       var_name(1:size))
         call nc_char_attrib(lat_id(drifter_no), 'units',
     >                       'degrees_north', var_name(1:size))
         range(1) = -90.
         range(2) = 90.
         call nc_range(lat_id(drifter_no), range, var_name(1:size))

         nc_status = NF_ENDDEF(nc_id)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 52)
         nc_status = NF_PUT_VAR_REAL(nc_id, lat_id(drifter_no),
     >                               variable)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 53)

         write(*,*)drifter_no,' Included variable   lat      / Latitude'


      else if (name(1:size) .eq. 'level') then

c        Level:

 221     format('level_',i6.6)
         write(var_name,221)drifter_no
         size = str_len(var_name)
         nc_status = NF_DEF_VAR(nc_id, var_name(1:size), NF_FLOAT,
     >                          1, var_1dim, level_id(drifter_no))
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 61)

 222     format('Vertical level, drifter no. ',i6.6)
         write(var_long_name,222)drifter_no
         long_size = str_len(var_long_name)
         call nc_char_attrib(level_id(drifter_no), 'long_name',
     >                       var_long_name(1:long_size),
     >                       var_name(1:size))
         call nc_char_attrib(level_id(drifter_no), 'units',
     >                       'm', var_name(1:size))
         call nc_miss_val_r(level_id(drifter_no), var_name(1:size))

         nc_status = NF_ENDDEF(nc_id)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 62)
         nc_status = NF_PUT_VAR_REAL(nc_id, level_id(drifter_no),
     >                               variable)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 63)

         write(*,*)drifter_no,' Included variable   level    / ',
     >                        'Vertical level'


      else if (name(1:size) .eq. 'time') then

c        Time:

 231     format('time_',i6.6)
         write(var_name,231)drifter_no
         size = str_len(var_name)
         nc_status = NF_DEF_VAR(nc_id, var_name(1:size), NF_FLOAT,
     >                          1, var_1dim, time_id(drifter_no))
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 71)

 232     format('Time, drifter no. ',i6.6)
         write(var_long_name,232)drifter_no
         long_size = str_len(var_long_name)
         call nc_char_attrib(time_id(drifter_no), 'long_name',
     >                       var_long_name(1:long_size),
     >                       var_name(1:size))
 233     format(i4.4, 5(a1,i2.2))
         write(date,233)reference_date(1),'-',reference_date(2),'-',
     >                  reference_date(3),' ',reference_date(4),':',
     >                  reference_date(5),':',reference_date(6)
         time_size = str_len(timeunit)
         nc_string = timeunit(1:time_size)//' since '// date
         time_size = str_len(nc_string)
         call nc_char_attrib(time_id(drifter_no), 'units',
     >                       nc_string(1:time_size), var_name(1:size))

         nc_status = NF_ENDDEF(nc_id)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 72)
         nc_status = NF_PUT_VAR_REAL(nc_id, time_id(drifter_no),
     >                               variable)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 73)

         write(*,*)drifter_no,' Included variable   time     / Time'


      else if (name(1:size) .eq. 'temp') then

c        Temperature:

 301     format('temp_',i6.6)
         write(var_name,301)drifter_no
         size = str_len(var_name)
         nc_status = NF_DEF_VAR(nc_id, var_name(1:size), NF_FLOAT,
     >                          1, var_1dim, temp_id(drifter_no))
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 101)

 302     format('Temperature, drifter no. ',i6.6)
         write(var_long_name,302)drifter_no
         long_size = str_len(var_long_name)
         call nc_char_attrib(temp_id(drifter_no), 'long_name',
     >                       var_long_name(1:long_size),
     >                       var_name(1:size))
         call nc_char_attrib(temp_id(drifter_no), 'units',
     >                       'degree_Kelvin', var_name(1:size))
         call nc_miss_val_r(level_id(drifter_no), var_name(1:size))

         nc_status = NF_ENDDEF(nc_id)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 102)
         nc_status = NF_PUT_VAR_REAL(nc_id, temp_id(drifter_no),
     >                               variable)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 103)

         write(*,*)drifter_no,' Included variable   temp     / ',
     >                        'Temperature'


      else if (name(1:size) .eq. 'salt') then

c        Salinity:

 311     format('salt_',i6.6)
         write(var_name,311)drifter_no
         size = str_len(var_name)
         nc_status = NF_DEF_VAR(nc_id, var_name(1:size), NF_FLOAT,
     >                          1, var_1dim, salt_id(drifter_no))
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 111)

 312     format('Salinity, drifter no. ',i6.6)
         write(var_long_name,312)drifter_no
         long_size = str_len(var_long_name)
         call nc_char_attrib(salt_id(drifter_no), 'long_name',
     >                       var_long_name(1:long_size),
     >                       var_name(1:size))
         call nc_char_attrib(salt_id(drifter_no), 'units', 'PSU',
     >                       var_name(1:size))
         call nc_miss_val_r(level_id(drifter_no), var_name(1:size))

         nc_status = NF_ENDDEF(nc_id)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 112)
         nc_status = NF_PUT_VAR_REAL(nc_id, salt_id(drifter_no),
     >                               variable)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 113)

         write(*,*)drifter_no,' Included variable   salt     / Salinity'


      else if (name(1:size) .eq. 'sgm_t') then

c        Sigma-t (density):

 321     format('sgm_t_',i6.6)
         write(var_name,321)drifter_no
         size = str_len(var_name)
         nc_status = NF_DEF_VAR(nc_id, var_name(1:size), NF_FLOAT,
     >                          1, var_1dim, sgm_t_id(drifter_no))
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 121)

 322     format('Sigma-t (density), drifter no. ',i6.6)
         write(var_long_name,322)drifter_no
         long_size = str_len(var_long_name)
         call nc_char_attrib(sgm_t_id(drifter_no), 'long_name',
     >                       var_long_name(1:long_size),
     >                       var_name(1:size))
         call nc_char_attrib(sgm_t_id(drifter_no), 'units', 'kg/(m**3)',
     >                       var_name(1:size))
         call nc_miss_val_r(level_id(drifter_no), var_name(1:size))

         nc_status = NF_ENDDEF(nc_id)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 122)
         nc_status = NF_PUT_VAR_REAL(nc_id, sgm_t_id(drifter_no),
     >                               variable)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 123)

         write(*,*)drifter_no,' Included variable   sgm_t    / ',
     >                        'Sigma-t (density)'


      else if (name(1:size) .eq. 'sst') then

c        Sea surface temperature:

 331     format('sst_',i6.6)
         write(var_name,331)drifter_no
         size = str_len(var_name)
         nc_status = NF_DEF_VAR(nc_id, var_name(1:size), NF_FLOAT,
     >                          1, var_1dim, sst_id(drifter_no))
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 131)

 332     format('Sea surface temperature, drifter no. ',i6.6)
         write(var_long_name,332)drifter_no
         long_size = str_len(var_long_name)
         call nc_char_attrib(sst_id(drifter_no), 'long_name',
     >                       var_long_name(1:long_size),
     >                       var_name(1:size))
         call nc_char_attrib(sst_id(drifter_no), 'units',
     >                       'degree_Kelvin', var_name(1:size))
         call nc_miss_val_r(level_id(drifter_no), var_name(1:size))

         nc_status = NF_ENDDEF(nc_id)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 132)
         nc_status = NF_PUT_VAR_REAL(nc_id, sst_id(drifter_no),
     >                               variable)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 133)

         write(*,*)drifter_no,' Included variable   sst      / ',
     >                        'Sea surface temperature'


      else if (name(1:size) .eq. 'sss') then

c        Sea surface salinity:

 341     format('sss_',i6.6)
         write(var_name,341)drifter_no
         size = str_len(var_name)
         nc_status = NF_DEF_VAR(nc_id, var_name(1:size), NF_FLOAT,
     >                          1, var_1dim, sss_id(drifter_no))
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 141)

 342     format('Sea surface salinity, drifter no. ',i6.6)
         write(var_long_name,342)drifter_no
         long_size = str_len(var_long_name)
         call nc_char_attrib(sss_id(drifter_no), 'long_name',
     >                       var_long_name(1:long_size),
     >                       var_name(1:size))
         call nc_char_attrib(sss_id(drifter_no), 'units', 'PSU',
     >                       var_name(1:size))
         call nc_miss_val_r(level_id(drifter_no), var_name(1:size))

         nc_status = NF_ENDDEF(nc_id)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 142)
         nc_status = NF_PUT_VAR_REAL(nc_id, sss_id(drifter_no),
     >                               variable)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 143)

         write(*,*)drifter_no,' Included variable   sss      / ',
     >                        'Sea surface salinity'


      else if (name(1:size) .eq. 'u') then

c        Velocity-x:

 401     format('u_',i6.6)
         write(var_name,401)drifter_no
         size = str_len(var_name)
         nc_status = NF_DEF_VAR(nc_id, var_name(1:size), NF_FLOAT,
     >                          1, var_1dim, u_id(drifter_no))
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 201)

 402     format('Velocity-x, drifter no. ',i6.6)
         write(var_long_name,402)drifter_no
         long_size = str_len(var_long_name)
         call nc_char_attrib(u_id(drifter_no), 'long_name',
     >                       var_long_name(1:long_size),
     >                       var_name(1:size))
         call nc_char_attrib(u_id(drifter_no), 'units', 'm/s',
     >                       var_name(1:size))
         call nc_miss_val_r(level_id(drifter_no), var_name(1:size))

         nc_status = NF_ENDDEF(nc_id)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 202)
         nc_status = NF_PUT_VAR_REAL(nc_id, u_id(drifter_no),
     >                               variable)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 203)

         write(*,*)drifter_no,' Included variable   u        / ',
     >                        'Velocity-x'


      else if (name(1:size) .eq. 'v') then

c        Velocity-y:

 411     format('v_',i6.6)
         write(var_name,411)drifter_no
         size = str_len(var_name)
         nc_status = NF_DEF_VAR(nc_id, var_name(1:size), NF_FLOAT,
     >                          1, var_1dim, v_id(drifter_no))
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 211)

 412     format('Velocity-y, drifter no. ',i6.6)
         write(var_long_name,412)drifter_no
         long_size = str_len(var_long_name)
         call nc_char_attrib(v_id(drifter_no), 'long_name',
     >                       var_long_name(1:long_size),
     >                       var_name(1:size))
         call nc_char_attrib(v_id(drifter_no), 'units', 'm/s',
     >                       var_name(1:size))
         call nc_miss_val_r(level_id(drifter_no), var_name(1:size))

         nc_status = NF_ENDDEF(nc_id)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 212)
         nc_status = NF_PUT_VAR_REAL(nc_id, v_id(drifter_no),
     >                               variable)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 212)

         write(*,*)drifter_no,' Included variable   v        / ',
     >                        'Velocity-y'


      else if (name(1:size) .eq. 'ssh') then

c        Sea surface height:

 501     format('ssh_',i6.6)
         write(var_name,501)drifter_no
         size = str_len(var_name)
         nc_status = NF_DEF_VAR(nc_id, var_name(1:size), NF_FLOAT,
     >                          1, var_1dim, ssh_id(drifter_no))
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 301)

 502     format('Sea surface height, drifter no. ',i6.6)
         write(var_long_name,502)drifter_no
         long_size = str_len(var_long_name)
         call nc_char_attrib(ssh_id(drifter_no), 'long_name',
     >                       var_long_name(1:long_size),
     >                       var_name(1:size))
         call nc_char_attrib(ssh_id(drifter_no), 'units', 'm',
     >                       var_name(1:size))
         call nc_miss_val_r(level_id(drifter_no), var_name(1:size))

         nc_status = NF_ENDDEF(nc_id)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 302)
         nc_status = NF_PUT_VAR_REAL(nc_id, ssh_id(drifter_no),
     >                               variable)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 303)

         write(*,*)drifter_no,' Included variable   ssh      / ',
     >                        'Sea surface height'


      end if


      go to 1000

c     'drifter_no' out of range:
 997  write(*,*)
      write(*,*)'Too many drifters!'
      write(*,*)' Max. no. of drifters, is specified in',
     >          ' "specs_drifter.inc" as: ',maxdrifters
      write(*,*)'Terminating in the subroutine "write_nc_drifter".'
      write(*,*)
      stop

c     'maxdrifters' out of range:
 998  write(*,*)
      write(*,*)'Too high limit for no. of drifters!'
      write(*,*)' Max. no. of drifters, is specified in',
     >          ' "specs_drifter.inc" as: ',maxdrifters
      write(*,*)' The upper limit in the present version of the',
     >          ' software is: ',maxdimension
      write(*,*)'Terminating in the subroutine "write_nc_drifter".'
      write(*,*)
      stop

c     'name' not recognized:
 999  write(*,*)
      write(*,*)'Unable to recognize name of variable: ',name
      write(*,*)' Not listed in "name_drifter.inc".'
      write(*,*)'Terminating in the subroutine "write_nc_drifter".'
      write(*,*)
      stop

 1000 return
      end


