
c     Subroutine on this file:
c      write_nc_header        - main subroutine for creating a netCDF header

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


c     - NOTES FOR "write_nc_header" -

c     **************
c     General notes:
c     **************

c     1. The path of the include file ('netdcf.inc') must be
c        included when compiling!; e.g.,
c          f77 -c -I/usr/local/netcdf-3.4/include write_nc_header.f

c     2. "write_nc_header" requires that the file which will contain
c        the header (see the file "specs.inc") does not already exist,
c        and that the file has not been opened outside "write_nc_header".

c     3. "write_nc_header" does not close the netCDF-file, but exits in
c        "data mode".


c     *********************************************
c     List of implemented parameters and variables:
c     *********************************************

c      (*1), (*2), (*3) : See comments below

c      parameter (short/long) type  dimensions  accuracy units      id
c      ^^^^^^^^^^^^^^^^^^^^^^ ^^^^  ^^^^^^^^^^  ^^^^^^^^ ^^^^^      ^^

c     COMPULSORY:
c      lon / Longitude        float (*1)        N/A      degrees    lon_id
c      lat / Latitude         float (*1)        N/A      degrees    lat_id
c      time / Time            float nt          N/A      (*1)       time_id

c     ... COMPULSORY IF VERTICAL RESOLUTION:
c      level / (*2)           int4  nz          (*3)     (*3)       level_id

c     ... COMPULSORY IF ICE:
c      ice_level / Ice levels int4  nz_ice      N/A      layer no.  ice_level_id

c     ANY OF:
c      temp / Temperature     int2  nx,ny,nz,nt 0.01     deg.
c                                                        Kelvin     temp_id
c      salt / Salinity        int2  nx,ny,nz,nt 0.01     PSU        salt_id
c      sst / Sea surface
c            temperature      int2  nx,ny,nt    0.01     deg.
c                                                        Kelvin     sst_id
c      sss / Sea surface
c            salinity         int2  nx,ny,nt    0.01     PSU        sss_id
c      sgm_t / Sigma-t
c              (density)      int2  nx,ny,nz,nt 0.01     kg/(m**3)  sgm_t_id
c      thck  / Layer thickness
c                             int2  nx,ny,nz,nt 0.1      m          thck_id
c      u / Velocity-x         int2  nx,ny,nz,nt 0.001    m/s        u_id
c      v / Velocity-y         int2  nx,ny,nz,nt 0.001    m/s        v_id
c      vol_x / Volume flux-x
c             (layers)        int2  nx,ny,nz,nt 0.1      (m**2)/s   vol_x_id
c      vol_y / Volume flux-y
c             (layers)        int2  nx,ny,nz,nt 0.1      (m**2)/s   vol_y_id
c      ice_frac / Ice cover
c                 (fraction)  int2  nx,ny,nt    0.001    none       ice_frac_id
c      ice_thck / Ice
c                 thickness   int2  nx,ny,
c                                   nz_ice,nt   0.001    m          ice_thck_id
c      u_ice / Ice velocity-x int2  nx,ny,nt    0.001    m/s        u_ice_id
c      v_ice / Ice velocity-y int2  nx,ny,nt    0.001    m/s        v_ice_id
c      vol_x_ice / Ice volume
c                  flux-x
c                  (layers)   int2  nx,ny,nt    0.001    (m**2)/s   vol_x_ice_id
c      vol_y_ice / Ice volume
c                  flux-y
c                  (layers)   int2  nx,ny,nt    0.001    (m**2)/s   vol_y_ice_id
c      ssh / Sea surface
c            height           int2  nx,ny,nt    0.001    m          ssh_id
c      topo / Bottom
c             topography      float nx,ny       N/A      m          topo_id


c     (*1):
c     'time' is set relative to 'reference_date', with units as specified
c     by 'timeunit'. These are both set in "specs.inc", and recognized
c     values of 'timeunit' are:
c      'seconds'
c      'minutes'
c      'hours'
c      'days'
c      'months'
c      'years'

c     (*2):
c     'lon'/'lat' dimensions depend on the grid ('sphere_on' in "specs.inc"):
c      parameter
c       (short/long name)   'sphere_on'  dimensions
c      ^^^^^^^^^^^^^^^^^^   ^^^^^^^^^^^  ^^^^^^^^^^
c      lon / Longitude      0            nx,ny
c      lon / Longitude      1            nx
c      lat / Latitude       0            nx,ny
c      lat / Latitude       1            ny

c     (*3):
c     The possible types of vertical levels ('leveltype' in "specs.inc") are:
c      type levels            accuracy  units
c      ^^^^ ^^^^^^            ^^^^^^^^  ^^^^^
c      1    Layers            N/A       none
c      2    z (positive)      1.0       m
c      3    z (negative)      1.0       m
c      4    sigma (positive)  0.001     none
c      5    sigma (negative)  0.001     none

c     Misc.:
c      int2: 2 byte integer (aka. "short")
c      int4: 4 byte integer (aka. "long")


      subroutine write_nc_header()


      implicit none

      include 'netcdf.inc'
      include 'id.inc'
      include 'specs.inc'

c     Local:
      integer dim_options
      parameter (dim_options = 9)
      integer nc_status,size,filesize,var_n_dim(dim_options),
     >        var_dim_ids(4,dim_options),option_no
      real range(2)
      character*19 date
      character*120 nc_string

c     Function:
      integer str_len


      filesize = str_len(filename)
      write(*,*)
      write(*,*)'================================================='
      write(*,*)' Attempting to create header for the netCDF file'
      write(*,*)'     ',filename(1:filesize)
      write(*,*)'================================================='


c     ************
c     netCDF Open:
c     ************

      nc_status = NF_CREATE(filename, NF_NOCLOBBER, nc_id)
      if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 1)


c     *************************
c     netCDF global attributes:
c     *************************

c     General information:
c     --------------------

      size = str_len(title)
      nc_status = NF_PUT_ATT_TEXT(nc_id, NF_GLOBAL, 'title',
     >                            size, title(1:size))
      if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 11)

      size = str_len(source)
      nc_status = NF_PUT_ATT_TEXT(nc_id, NF_GLOBAL, 'source',
     >                            size, source(1:size))
      if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 12)

      size = str_len(method)
      nc_status = NF_PUT_ATT_TEXT(nc_id, NF_GLOBAL, 'method',
     >                            size, method(1:size))
      if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 13)

      size = str_len(reference)
      nc_status = NF_PUT_ATT_TEXT(nc_id, NF_GLOBAL, 'reference',
     >                            size, reference(1:size))
      if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 14)

      size = str_len(web_reference)
      nc_status = NF_PUT_ATT_TEXT(nc_id, NF_GLOBAL, 'web_reference',
     >                            size, web_reference(1:size))
      if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 15)

      size = str_len(history)
      nc_status = NF_PUT_ATT_TEXT(nc_id, NF_GLOBAL, 'history',
     >                           size,  history(1:size))
      if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 16)

      size = str_len(author)
      nc_status = NF_PUT_ATT_TEXT(nc_id, NF_GLOBAL, 'author',
     >                            size, author(1:size))
      if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 17)

      size = str_len(email)
      nc_status = NF_PUT_ATT_TEXT(nc_id, NF_GLOBAL, 'email',
     >                            size, email(1:size))
      if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 18)



c     ******************
c     netCDF dimensions:
c     ******************

      write(*,*)
      write(*,*)'Dimensions:'

      if (sphere_on) then

         nc_status = NF_DEF_DIM(nc_id, 'lon', nx, x_id)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 21)

         nc_status = NF_DEF_DIM(nc_id, 'lat', ny, y_id)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 22)

      else

         nc_status = NF_DEF_DIM(nc_id, 'x', nx, x_id)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 21)

         nc_status = NF_DEF_DIM(nc_id, 'y', ny, y_id)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 22)

      end if

      write(*,*)' x:     ',nx
      write(*,*)' y:     ',ny

      if (nz .gt. 0) then
         nc_status = NF_DEF_DIM(nc_id, 'z', nz, z_id)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 23)
         write(*,*)' z:     ',nz
      end if

      if (nz_ice .gt. 0) then
         nc_status = NF_DEF_DIM(nc_id, 'z_ice', nz_ice, z_ice_id)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 24)
         write(*,*)' z_ice: ',nz_ice
      end if

      nc_status = NF_DEF_DIM(nc_id, 'time', NF_UNLIMITED, t_id)
      if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 25)
      write(*,*)' time:  NF_UNLIMITED'


c     Set arrays with dimension id-s:

c     option 1: x (longitude)
      var_n_dim(1) = 1
      var_dim_ids(1,1) = x_id

c     option 2: y (latitude)
      var_n_dim(2) = 1
      var_dim_ids(1,2) = y_id

c     option 3: z
      var_n_dim(3) = 1
      var_dim_ids(1,3) = z_id

c     option 4: z_ice
      var_n_dim(4) = 1
      var_dim_ids(1,4) = z_ice_id

c     option 5: t
      var_n_dim(5) = 1
      var_dim_ids(1,5) = t_id

c     option 6: x,y
      var_n_dim(6) = 2
      var_dim_ids(1,6) = x_id
      var_dim_ids(2,6) = y_id

c     option 7: x,y,t
      var_n_dim(7) = 3
      var_dim_ids(1,7) = x_id
      var_dim_ids(2,7) = y_id
      var_dim_ids(3,7) = t_id

c     option 8: x,y,z,t
      var_n_dim(8) = 4
      var_dim_ids(1,8) = x_id
      var_dim_ids(2,8) = y_id
      var_dim_ids(3,8) = z_id
      var_dim_ids(4,8) = t_id

c     option 9: x,y,z,t
      var_n_dim(9) = 4
      var_dim_ids(1,9) = x_id
      var_dim_ids(2,9) = y_id
      var_dim_ids(3,9) = z_ice_id
      var_dim_ids(4,9) = t_id


c     *****************
c        ***********

c     *****************
c     netCDF Variables:
c     *****************


c     Grid specification:
c     %%%%%%%%%%%%%%%%%%%


      write(*,*)
      write(*,*)'Grid specification:'

      if (sphere_on) then

c        Longitude:

         option_no = 1
         nc_status = NF_DEF_VAR(nc_id, 'lon', NF_FLOAT,
     >                          var_n_dim(option_no),
     >                          var_dim_ids(1,option_no), lon_id)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 31)
         call nc_char_attrib(lon_id, 'long_name', 'Longitude', 'lon')
         call nc_char_attrib(lon_id, 'units', 'degrees_east', 'lon')
         range(1) = -360.
         range(2) = 360.
         call nc_range(lon_id, range, 'lon')
         write(*,*)' Included variable   lon      / Longitude'

c        Latitude:

         option_no = 2
         nc_status = NF_DEF_VAR(nc_id, 'lat', NF_FLOAT,
     >                          var_n_dim(option_no),
     >                          var_dim_ids(1,option_no), lat_id)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 41)
         call nc_char_attrib(lat_id, 'long_name', 'Latitude', 'lat')
         call nc_char_attrib(lat_id, 'units', 'degrees_north', 'lon')
         range(1) = -90.
         range(2) = 90.
         call nc_range(lat_id, range, 'lat')
         write(*,*)' Included variable   lat      / Latitude'

      else

         option_no = 6

c        Longitude:

         nc_status = NF_DEF_VAR(nc_id, 'lon', NF_FLOAT,
     >                          var_n_dim(option_no),
     >                          var_dim_ids(1,option_no), lon_id)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 31)
         call nc_char_attrib(lon_id, 'long_name', 'Longitude', 'lon')
         call nc_char_attrib(lon_id, 'units', 'degrees_east', 'lon')
         range(1) = -360.
         range(2) = 360.
         call nc_range(lon_id, range, 'lon')
         write(*,*)' Included variable   lon      / Longitude'

c        Latitude:

         nc_status = NF_DEF_VAR(nc_id, 'lat', NF_FLOAT,
     >                          var_n_dim(option_no),
     >                          var_dim_ids(1,option_no), lat_id)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 41)
         call nc_char_attrib(lat_id, 'long_name', 'Latitude', 'lat')
         call nc_char_attrib(lat_id, 'units', 'degrees_north', 'lon')
         range(1) = -90.
         range(2) = 90.
         call nc_range(lat_id, range, 'lat')
         write(*,*)' Included variable   lat      / Latitude'

      end if


      if (nz .gt. 0)
c        Levels:
     >      call write_nc_intgr_header
     >              ('level',NF_INT,dim_options,var_n_dim,var_dim_ids,
     >               level_offset,level_scale,level_id)


      if (nz_ice .gt. 0)
c        Ice levels:
     >      call write_nc_intgr_header
     >              ('ice_level',NF_INT,
     >               dim_options,var_n_dim,var_dim_ids,
     >               ice_level_offset,ice_level_scale,ice_level_id)


c     Time:
      option_no = 5
      nc_status = NF_DEF_VAR(nc_id, 'time', NF_FLOAT,
     >                          var_n_dim(option_no),
     >                          var_dim_ids(1,option_no), time_id)
      if (nc_status .ne. NF_NOERR) call handle_err(nc_status,81)
      call nc_char_attrib(time_id, 'long_name', 'Time', 'time')

 101  format(i4.4, 5(a1,i2.2))
      write(date,101)reference_date(1),'-',reference_date(2),'-',
     >               reference_date(3),' ',reference_date(4),':',
     >               reference_date(5),':',reference_date(6)
      size = str_len(timeunit)
      nc_string = timeunit(1:size)//' since '// date
      size = str_len(nc_string)
      call nc_char_attrib(time_id, 'units', nc_string(1:size), 'time')
      size = str_len(date)
      call nc_char_attrib(time_id, 'origin', date(1:size), 'time')
      write(date,101)timestep(1),'-',timestep(2),'-',
     >               timestep(3),' ',timestep(4),':',
     >               timestep(5),':',timestep(6)
      call nc_char_attrib(time_id, 'delta_t', date, 'time')
      size = str_len(timemethod)
      call nc_char_attrib(time_id, 'method', timemethod(1:size), 'time')
      write(*,*)' Included variable   time     / Time'


c     Hydrography:
c     %%%%%%%%%%%%

      write(*,*)
      write(*,*)'Hydrography:'

      if (nz .gt. 0) then

c        Temperature:
         if (temp_on)
     >      call write_nc_intgr_header
     >              ('temp',NF_SHORT,dim_options,var_n_dim,var_dim_ids,
     >               temp_offset,temp_scale,temp_id)

c        Temperature (Kelvin):
         if (temp_K_on)
     >      call write_nc_intgr_header
     >              ('temp_K',NF_SHORT,
     >               dim_options,var_n_dim,var_dim_ids,
     >               temp_K_offset,temp_K_scale,temp_K_id)

c        Salinity:
         if (salt_on)
     >      call write_nc_intgr_header
     >              ('salt',NF_SHORT,dim_options,var_n_dim,var_dim_ids,
     >               salt_offset,salt_scale,salt_id)

c        Sigma-t (density):
         if (sgm_t_on)
     >      call write_nc_intgr_header
     >              ('sgm_t',NF_SHORT,dim_options,var_n_dim,var_dim_ids,
     >               sgm_t_offset,sgm_t_scale,sgm_t_id)

      end if

c     Sea surface temperature:
      if (sst_on)
     >   call write_nc_intgr_header
     >           ('sst',NF_SHORT,dim_options,var_n_dim,var_dim_ids,
     >            sst_offset,sst_scale,sst_id)

c     Sea surface temperature:
      if (sst_K_on)
     >   call write_nc_intgr_header
     >           ('sst_K',NF_SHORT,dim_options,var_n_dim,var_dim_ids,
     >            sst_K_offset,sst_K_scale,sst_K_id)

c     Sea surface salinity:
      if (sss_on)
     >   call write_nc_intgr_header
     >           ('sss',NF_SHORT,dim_options,var_n_dim,var_dim_ids,
     >            sss_offset,sss_scale,sss_id)

c     Layer thickess:
      if (nz .gt. 0 .and. leveltype .eq. 1 .and. thck_on)
     >   call write_nc_intgr_header
     >           ('thck',NF_SHORT,dim_options,var_n_dim,var_dim_ids,
     >            thck_offset,thck_scale,thck_id)


c     Kinematics:
c     %%%%%%%%%%%

      write(*,*)
      write(*,*)'Kinematics:'

      if (nz .gt. 0) then

c        Velocity-x:
         if (u_on)
     >      call write_nc_intgr_header
     >              ('u',NF_SHORT,dim_options,var_n_dim,var_dim_ids,
     >               u_offset,u_scale,u_id)

c        Velocity-y:
         if (v_on)
     >      call write_nc_intgr_header
     >              ('v',NF_SHORT,dim_options,var_n_dim,var_dim_ids,
     >               v_offset,v_scale,v_id)

c        Volume flux-x (layers):
         if (vol_x_on)
     >      call write_nc_intgr_header
     >              ('vol_x',NF_SHORT,dim_options,var_n_dim,var_dim_ids,
     >               vol_x_offset,vol_x_scale,vol_x_id)

c        Volume flux-y (layers):
         if (vol_y_on)
     >      call write_nc_intgr_header
     >              ('vol_y',NF_SHORT,dim_options,var_n_dim,var_dim_ids,
     >               vol_y_offset,vol_y_scale,vol_y_id)

      end if


      if (nz_ice .gt. 0) then

c        Ice:
c        %%%%

         write(*,*)
         write(*,*)'Ice:'

c        Ice cover (fraction):
         if (ice_frac_on)
     >      call write_nc_intgr_header
     >              ('ice_frac',NF_SHORT,
     >               dim_options,var_n_dim,var_dim_ids,
     >               ice_frac_offset,ice_frac_scale,ice_frac_id)

c        Ice thickness:
         if (ice_thck_on)
     >      call write_nc_intgr_header
     >              ('ice_thck',NF_SHORT,
     >               dim_options,var_n_dim,var_dim_ids,
     >               ice_thck_offset,ice_thck_scale,ice_thck_id)

c        Ice velocity-x:
         if (u_ice_on)
     >      call write_nc_intgr_header
     >              ('u_ice',NF_SHORT,dim_options,var_n_dim,var_dim_ids,
     >               u_ice_offset,u_ice_scale,u_ice_id)

c        Ice velocity-y:
         if (v_ice_on)
     >      call write_nc_intgr_header
     >              ('v_ice',NF_SHORT,dim_options,var_n_dim,var_dim_ids,
     >               v_ice_offset,v_ice_scale,v_ice_id)

c        Ice volume flux-x (layers):
         if (vol_x_ice_on)
     >      call write_nc_intgr_header
     >              ('vol_x_ice',NF_SHORT,
     >               dim_options,var_n_dim,var_dim_ids,
     >               vol_x_ice_offset,vol_x_ice_scale,vol_x_ice_id)

c        Ice volume flux-y (layers):
         if (vol_y_ice_on)
     >      call write_nc_intgr_header
     >              ('vol_y_ice',NF_SHORT,
     >               dim_options,var_n_dim,var_dim_ids,
     >               vol_y_ice_offset,vol_y_ice_scale,vol_y_ice_id)

      end if


c     Topography:
c     %%%%%%%%%%%

      write(*,*)
      write(*,*)'Topography:'

c     Sea surface height:
      if (ssh_on)
     >   call write_nc_intgr_header
     >           ('ssh',NF_SHORT,dim_options,var_n_dim,var_dim_ids,
     >            ssh_offset,ssh_scale,ssh_id)

c     Bottom topography:
      if (topo_on) then
         option_no = 6
         nc_status = NF_DEF_VAR(nc_id, 'topo', NF_FLOAT,
     >                          var_n_dim(option_no),
     >                          var_dim_ids(1,option_no), topo_id)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status,411)
         call nc_char_attrib(topo_id, 'long_name', 'Bottom topography',
     >                       'topo')
         call nc_char_attrib(topo_id, 'units', 'm', 'topo')
         call nc_miss_val_r(topo_id, 'topo')
         write(*,*)' Included variable   topo     / Bottom topography'
      end if


c     ***********************
c          *************

c     ***********************
c     netCDF end definitions:
c     ***********************

      nc_status = NF_ENDDEF(nc_id)
      if (nc_status .ne. NF_NOERR) call handle_err(nc_status,53)

      write(*,*)
      write(*,*)'========================================='
      write(*,*)' Done creating netCDF header on the file'
      write(*,*)'     ',filename(1:filesize)
      write(*,*)' The file has been set in "data mode".'
      write(*,*)'========================================='
      write(*,*)


      end
