
ccc   Subroutines on "dates.f":
ccc    diff_time        - computes the difference between two dates,
ccc                       in days, minutes, hours and (decimal) seconds
ccc    add_time         - adds a time offset [in days, minutes, hours
ccc                       and (decimal) seconds] to a date
ccc    check_date       - checks that a given date is valid
ccc    adjust_d_time    - modifies a time offset [in days, hours,
ccc                       minutes and (decimal) seconds] so that all
ccc                       elements are within their natural ranges,
ccc                       and so that all non-zero elements have the
ccc                       same sign
ccc    offset2date      - adds a time offset in recognized units
ccc                       ('years', 'months', 'days', 'hours',
ccc                        'minutes', 'seconds') to a date
ccc
ccc   Functions on "dates.f":
ccc    unit2unit        - converts an interval of one unit to the same
ccc                       interval with a different unit, and returns
ccc                       the latter
ccc    m_days, n_days   - functions that return the no. of days in a
ccc                       particular month or year, respectively
ccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc



c     ********************
c     General description:
c     ********************

c     The subroutine "diff_time" computes the difference between two
c     dates, which may be positive or negative. The input dates must
c     be valid. The difference is returned as an offset
c     in days, minutes, hours and (decimal) seconds,
c     where hours<24, minutes<60, seconds<60.0


c     ***********************
c     Input/output variables:
c     ***********************

c      climate        (l)  - switch, if (.true.) use "climate years"
c                             (12 months with 30 days in each month);
c                             if (.false.) use calendar years;
c      first_date     (iA) - the first date (see note below), format:
c                              first_date(1) - year
c                              first_date(2) - month
c                              first_date(3) - day
c                              first_date(4) - hour
c                              first_date(5) - minute
c      first_sec      (r)  - seconds to add to 'first_date', to get the
c                             exact time
c      last_date      (iA) - the last date (see note below), format:
c                              last_date(1) - year
c                              last_date(2) - month
c                              last_date(3) - day
c                              last_date(4) - hour
c                              last_date(5) - minute
c      last_sec       (r)  - seconds to add to 'last_date', to get the
c                             exact time
c      d_time         (iA) - output time
c                             (from 'first_date' to 'last_date'), format:
c                               d_time(1) - days
c                               d_time(2) - hours (<24)
c                               d_time(3) - minutes (<60)
c      d_sec          (r)  - output, no. of seconds to add to 'd_time'
c                             to get the exact difference

c     NOTE:
c      1. 'first_date'/'first_sec' may well refer to a time later than
c         'last_date' /'last_sec'. Then, the output values will be
c         negative.
c      2. The non-zero entries in 'd_time' and 'd_sec' will all have
c         the same sign.

c     Above:
c      l  - logical          [ (.false.)/0 or (.true.)/nonzero ]
c      iA - integer array    [ 4 byte, aka. "long" ]
c      r  - real variable    [ 4 byte, aka. "float" ]


      subroutine diff_time(climate,first_date,first_sec,
     >                     last_date,last_sec,d_time,d_sec)

      implicit none

c     Input:
      integer first_date(5),last_date(5)
      real first_sec,last_sec
      logical climate

c     Output:
      integer d_time(3)
      real d_sec

c     Local:
      integer month_days,annual_days,year,month

c     Function:
      integer m_days,n_days


c     Check 'first_date'/'first_sec':
      call check_date(climate,first_date,first_sec)
c     Check 'last_date'/'last_sec':
      call check_date(climate,last_date,last_sec)


c     Hours, minutes, seconds:
      d_time(2) = last_date(4) - first_date(4)
      d_time(3) = last_date(5) - first_date(5)
      d_sec     = last_sec     - first_sec

c     Days:

c     Back up to Jan. 1.:
      d_time(1) = - first_date(3) + 1
      month = first_date(2)
      do while (month .gt. 1)
        month = month - 1
        month_days = m_days(climate,month,year)
        d_time(1) = d_time(1) - month_days
      end do

c     Step forward/backward, years:
      year = first_date(1)
c     Forward:
      do while (year .lt. last_date(1))
        annual_days = n_days(climate,year)
        d_time(1) = d_time(1) + annual_days
        year = year + 1
      end do
c     Backward:
      do while (year .gt. last_date(1))
        year = year - 1
        annual_days = n_days(climate,year)
        d_time(1) = d_time(1) - annual_days
      end do

c     Step forward, months:
      do while (month .lt. last_date(2))
        month_days = m_days(climate,month,year)
        d_time(1) = d_time(1) + month_days
        month = month + 1
      end do

c     Remaining days:
      d_time(1) = d_time(1) + last_date(3) - 1

c     Adjust offset to natural range:
      call adjust_d_time(d_time,d_sec)

      return
      end



c     ********************
c     General description:
c     ********************

c     The subroutine "add_time" adds a time offset to a date.
c     The input values for the offset are supplied as no. of days,
c     minutes, hours and (decimal) seconds (without restricitions).
c     The subroutine is based on the DNMI library subroutine "vtime",
c     by Anstein Foss.


c     ***********************
c     Input/output variables:
c     ***********************

c      climate        (l)  - switch, if (.true.) use "climate years"
c                             (12 months with 30 days in each month);
c                             if (.false.) use calendar years;
c      date           (iA) - input: original date; output: updated
c                             format:
c                              date(1) - year
c                              date(2) - month
c                              date(3) - day
c                              date(4) - hour
c                              date(5) - minute
c      sec            (r)  - seconds to add to 'date', to get the exact
c                             time (input and output as 'date' above)
c      d_time         (iA) - days, hours, minutes to add to
c                             input 'date'/'sec' to get output 'date'/'sec'
c                             format:
c                              d_time(1) - days
c                              d_time(2) - hours
c                              d_time(3) - minutes
c      d_sec          (r)  - seconds to add to input 'date'/'sec'
c                             to get output 'date'/'sec'

c     NOTE:
c      1. 'd_time'/'d_sec" may contain positive values, negative values,
c         or a combination of positive and negative values.
c      2. There are no restrictions on magnitudes
c         (i.e. |d_time(2)| may well be 24 or larger)

c     Above:
c      l  - logical          [ (.false.)/0 or (.true.)/nonzero ]
c      iA - integer array    [ 4 byte, aka. "long" ]
c      r  - real variable    [ 4 byte, aka. "float" ]


      subroutine add_time(climate,date,sec,d_time,d_sec)

      implicit none

c     Input:
      integer d_time(3)
      real d_sec
      logical climate

c     Input/output:
      integer date(5)
      real sec

c     Local:
      integer days,hours,minutes,month_days
      logical frwrd

c     Function:
      integer m_days

c     Check 'date'/'sec':
      call check_date(climate,date,sec)

c     Adjust 'd_time'/'d_sec':
      call adjust_d_time(d_time,d_sec)

      days    = d_time(1)
      hours   = d_time(2)
      minutes = d_time(3)

      if (d_time(3) .gt. 0) then
         frwrd = .true.
      else if (d_time(2) .gt. 0) then
         frwrd = .true.
      else if (d_time(1) .gt. 0) then
         frwrd = .true.
      else if (d_sec .gt. 0.) then
         frwrd = .true.
      else
         frwrd = .false.
      end if

      if (frwrd) then

c        seconds:
         sec = sec + d_sec
         if (sec .ge. 60.) then
            sec = sec - 60.
            minutes = minutes + 1
         end if

c        minutes:
         date(5) = date(5) + minutes
         if (date(5) .ge. 60) then
            date(5) = date(5) - 60
            hours = hours + 1
         end if

c        hours:
         date(4) = date(4) + hours
         if (date(4) .ge. 24) then
            date(4) = date(4) - 24
            days = days + 1
         end if

c        days, months, years:
         month_days = m_days(climate,date(2),date(1))

         if (date(3)+days .le. month_days) then

            date(3) = date(3) + days

         else

            days = days - (month_days - date(3))
            date(3) = 1
            date(2) = date(2) + 1
            if (date(2) .eq. 13) then
               date(2) = 1
               date(1) = date(1) + 1
            end if

            month_days = m_days(climate,date(2),date(1))
            do while (days .gt. month_days)
               days = days - month_days
               date(2) = date(2) + 1
               if (date(2) .eq. 13) then
                  date(2) = 1
                  date(1) = date(1) + 1
               end if
               month_days = m_days(climate,date(2),date(1))
            end do

            date(3) = days

         end if

      else

c        seconds:
         sec = sec + d_sec
         if (sec .lt. 0.) then
            sec = sec + 60.
            minutes = minutes - 1
         end if

c        minutes:
         date(5) = date(5) + minutes
         if (date(5) .lt. 0) then
            date(5) = date(5) + 60
            hours = hours - 1
         end if

c        hours:
         date(4) = date(4) + hours
         if (date(4) .lt. 0) then
            date(4) = date(4) + 24
            days = days - 1
         end if

c        days, months, years:

         if (date(3)+days .ge. 1) then

            date(3) = date(3) + days

         else

            days = days + date(3)
            date(2) = date(2) - 1
            if (date(2) .eq. 0) then
               date(2) = 12
               date(1) = date(1) - 1
            end if

            month_days = m_days(climate,date(2),date(1))
            do while (month_days+days .le. 0)
               days = days + month_days
               date(2) = date(2) - 1
               if (date(2) .eq. 0) then
                  date(2) = 12
                  date(1) = date(1) - 1
               end if
               month_days = m_days(climate,date(2),date(1))
            end do

            date(3) = month_days + days

         end if

      end if

      return
      end



c     ********************
c     General description:
c     ********************

c     The subroutine "check_date" checks that a given date is valid.
c     If the date is valid, the subroutine returns with no action taken.
c     If the date is invalid, an error message is written, and the
c     program terminates.


c     ***********************
c     Input/output variables:
c     ***********************

c      climate        (l)  - switch, if (.true.) use "climate years"
c                             (12 months with 30 days in each month);
c                             if (.false.) use calendar years;
c      date           (iA) - date to check, format:
c                              date(1) - year
c                              date(2) - month
c                              date(3) - day
c                              date(4) - hour
c                              date(5) - minute
c      sec            (r)  - seconds to add to 'date', to get the exact
c                             time

c     Above:
c      l  - logical          [ (.false.)/0 or (.true.)/nonzero ]
c      iA - integer array    [ 4 byte, aka. "long" ]
c      r  - real variable    [ 4 byte, aka. "float" ]


      subroutine check_date(climate,date,sec)

      implicit none

c     Input:
      integer date(5)
      real sec
      logical climate

c     Local:
      integer month_days

c     Function:
      integer m_days


      if (date(2) .lt. 1 .or. date(3) .lt. 1 .or.
     >    date(4) .lt. 0 .or. date(5) .lt. 0 .or. sec .lt. 0.)
     >   go to 999

      month_days = m_days(climate,date(2),date(1))

      if (date(2) .gt. 12 .or. date(3) .gt. month_days .or.
     >    date(4) .ge. 24 .or. date(5) .ge. 60 .or. sec .ge. 60.)
     >   go to 999

      go to 1000

 999  write(*,*)
 101  format('           -----------> OUT OF RANGE!\n')
      write(*,*)'Error in the subroutine "check_date".'
      write(*,*)' Error in date format:'

      write(*,*)'  Month:   range: [1 , 12 ]; is ',date(2)
      if (date(2) .lt. 1 .or. date(2) .gt. 12) write(*,101)

 102  format(1x,a23,i2,a6,$)
      write(*,102)'  Day:     range: [1 , ',month_days,' ]; is'
      write(*,*)date(3)
      if (date(3) .lt. 1 .or. date(3) .gt. month_days) write(*,101)

      write(*,*)'  Hour:    range: [0 , 23 ]; is ',date(4)
      if (date(4) .lt. 0 .or. date(4) .gt. 23) write(*,101)

      write(*,*)'  Minute:  range: [0 , 59 ]: is ',date(5)
      if (date(5) .lt. 0 .or. date(5) .gt. 59) write(*,101)

      write(*,*)'  Second:  range: [0., 60.>; is ',sec
      if (sec .lt. 0. .or. sec .ge. 60.) write(*,101)

      write(*,*)'Terminates.'
      write(*,*)
      stop

 1000 return
      end



c     ********************
c     General description:
c     ********************

c     The subroutine "adjust_d_time" modifies a time offset in days,
c     hours, minutes and (decimal) seconds so that the output correspond
c     to the same interval, and all output non-zero elements have the
c     same sign. Moreover, the elements corresponding to hours, minutes
c     and seconds are returned within their natural ranges.


c     ***********************
c     Input/output variables:
c     ***********************

c      d_time         (iA) - time offset to modify, format:
c                              d_time(1) - days
c                              d_time(2) - hours
c                              d_time(3) - minutes
c                             input: no restrictions
c                             output, negative:
c                              d_time(1) in <   , 0]
c                              d_time(2) in [-23, 0]
c                              d_time(3) in [-59, 0]
c                             output, positive:
c                              d_time(1) in [0,   >
c                              d_time(2) in [0, 23]
c                              d_time(3) in [0, 59]
c      d_sec          (r)  - seconds to add to 'd_time', to get the exact
c                             time offset
c                             input: no restrictions
c                             output, negative:
c                              d_sec in <-60., 0.]
c                             output, positive:
c                              d_sec in [0., 60.>

c     Above:
c      iA - integer array    [ 4 byte, aka. "long" ]
c      r  - real variable    [ 4 byte, aka. "float" ]


      subroutine adjust_d_time(d_time,d_sec)

      implicit none

c     Input/output:
      integer d_time(3)
      real d_sec

c     Local:
      real*8 days

      days = dble(d_time(1)) + dble(d_time(2))/dble(24.)
     >                       + dble(d_time(3))/dble(24.*60.)
     >                       + dble(d_sec)    /dble(24.*3600.)


      if (days .gt. dble(0.)) then

c        Seconds -> minutes:
         do while (d_sec .lt. 0.)
            d_sec = d_sec + 60.
            d_time(3) = d_time(3) - 1
         end do
         do while (d_sec .ge. 60.)
            d_sec = d_sec - 60.
            d_time(3) = d_time(3) + 1
         end do

c        Minutes -> hours:
         do while (d_time(3) .lt. 0)
            d_time(3) = d_time(3) + 60
            d_time(2) = d_time(2) - 1
         end do
         do while (d_time(3) .ge. 60)
            d_time(3) = d_time(3) - 60
            d_time(2) = d_time(2) + 1
         end do

c        Hours -> days:
         do while (d_time(2) .lt. 0)
            d_time(2) = d_time(2) + 24
            d_time(1) = d_time(1) - 1
         end do
         do while (d_time(2) .ge. 24)
            d_time(2) = d_time(2) - 24
            d_time(1) = d_time(1) + 1
         end do

      else

c        Seconds -> minutes:
         do while (d_sec .gt. 0.)
            d_sec = d_sec - 60.
            d_time(3) = d_time(3) + 1
         end do
         do while (d_sec .le. -60.)
            d_sec = d_sec + 60.
            d_time(3) = d_time(3) - 1
         end do

c        Minutes -> hours:
         do while (d_time(3) .gt. 0)
            d_time(3) = d_time(3) - 60
            d_time(2) = d_time(2) + 1
         end do
         do while (d_time(3) .le. -60)
            d_time(3) = d_time(3) + 60
            d_time(2) = d_time(2) - 1
         end do

c        Hours -> days:
         do while (d_time(2) .gt. 0)
            d_time(2) = d_time(2) - 24
            d_time(1) = d_time(1) + 1
         end do
         do while (d_time(2) .le. -24)
            d_time(2) = d_time(2) + 24
            d_time(1) = d_time(1) - 1
         end do

      end if

      return
      end



c     ********************
c     General description:
c     ********************

c     The subroutine "offset2date" adds a time offset in recognized
c     units to a date. If the unit is 'years' or 'months', "climate
c     years" (12 30-day months) are used, otherwise, the value of
c     'climate' is invoked.


c     ***********************
c     Input/output variables:
c     ***********************

c      climate        (l)  - switch, if (.true.) use "climate years"
c                             (12 months with 30 days in each month);
c                             if (.false.) use calendar years;
c                             disregarded if 'unit' is 'years' or
c                             'months' (then, the use of "climate
c                             years" is forced)
c      unit           (s)  - a character string representing the unit
c                             of the time offset, implemented:
c                              'years', 'months'   'days'
c                              'hours', 'minutes', 'seconds'
c      origin_date    (iA) - the date of zero offset, format:
c                              date(1) - year
c                              date(2) - month
c                              date(3) - day
c                              date(4) - hour
c                              date(5) - minute
c      origin_sec     (r)  - seconds to add to 'origin_date', to get
c                             the exact date of zero offset
c      dt             (r)  - time offset, given in units as specified
c                             (see 'unit' above)
c      date           (iA) - output, the resulting date when 'dt' has
c                              been added to 'origindate'/'origin_sec'
c      sec            (r)  - output, seconds to add to 'date' to get
c                             the exact updated time

c     NOTE:
c      'dt" can not be negative (then, 'origindate'/'origin_sec' is
c       not the "origin date"

c     Above:
c      l  - logical          [ (.false.)/0 or (.true.)/nonzero ]
c      s  - string           [ character*(*) ]
c      iA - integer array    [ 4 byte, aka. "long" ]
c      r  - real variable    [ 4 byte, aka. "float" ]


      subroutine offset2date(climate,unit,origin_date,origin_sec,dt,
     >                       date,sec)

      implicit none

c     Input:
      integer origin_date(5)
      real origin_sec,dt
      logical climate
      character*(*) unit

c     Output:
      integer date(5)
      real sec

c     Local:
      integer d_time(3),i,first,last,length
      real d_sec
      real*8 value
      logical clmt

c     Functions:
      integer str_pos,str_len
      character null_chrctr


c     Check:
      if (dt .lt. 0.) go to 999

c     Initialize:
      value = dble(dt)
      length = str_len(unit)

c     Set position of first and last relevant characters in 'unit'
      first = 1
      do while (unit(first:first) .eq. ' ')
         first = first + 1
      end do
      last = str_pos(unit,first+1,' ') - 1
      if (last .lt. first) last = length
c      if (unit(last:last) .eq. ' ' .or.
c     >    unit(last:last) .eq. null_chrctr()) last = last - 1

      if (unit(first:last) .eq. 'years') then
c        Disregarding 'climate'
         clmt = .true.
         value = dble(360)*value
         d_time(1) = int(value)
         value = value - dble(int(value))
         value = dble(24)*value
         d_time(2) = int(value)
         value = value - dble(int(value))
         value = dble(60)*value
         d_time(3) = int(value)
         value = value - dble(int(value))
         d_sec = 60.*float(value)
      else if (unit(first:last) .eq. 'months') then
c        Disregarding 'climate'
         clmt = .true.
         value = dble(30)*value
         d_time(1) = int(value)
         value = value - dble(int(value))
         value = dble(24)*value
         d_time(2) = int(value)
         value = value - dble(int(value))
         value = dble(60)*value
         d_time(3) = int(value)
         value = value - dble(int(value))
         d_sec = 60.*float(value)
      else if (unit(first:last) .eq. 'days') then
         clmt = climate
         d_time(1) = int(value)
         value = value - dble(int(value))
         value = dble(24)*value
         d_time(2) = int(value)
         value = value - dble(int(value))
         value = dble(60)*value
         d_time(3) = int(value)
         value = value - dble(int(value))
         d_sec = 60.*float(value)
      else if (unit(first:last) .eq. 'hours') then
         clmt = climate
         value = value/dble(24)
         d_time(1) = int(value)
         value = value - dble(int(value))
         value = dble(24.)*value
         d_time(2) = int(value)
         value = value - dble(int(value))
         value = dble(60)*value
         d_time(3) = int(value)
         value = value - dble(int(value))
         d_sec = 60.*float(value)
      else if (unit(first:last) .eq. 'minutes') then
         clmt = climate
         value = value/dble(24*60)
         d_time(1) = int(value)
         value = value - dble(int(value))
         value = dble(24)*value
         d_time(2) = int(value)
         value = value - dble(int(value))
         value = dble(60)*value
         d_time(3) = int(value)
         value = value - dble(int(value))
         d_sec = 60.*float(value)
      else if (unit(first:last) .eq. 'seconds') then
         clmt = climate
         value = value/dble(24*60*60)
         d_time(1) = int(value)
         value = value - dble(int(value))
         value = dble(24)*value
         d_time(2) = int(value)
         value = value - dble(int(value))
         value = dble(60)*value
         d_time(3) = int(value)
         value = value - dble(int(value))
         d_sec = 60.*float(value)
      else
         go to 998
      end if

c     Initialize, update 'date', 'sec':
      do i = 1,5
         date(i) = origin_date(i)
      end do
      sec = origin_sec
      call add_time(clmt,date,sec,d_time,d_sec)

      go to 1000

 998  write(*,*)
      write(*,*)'Error in the subroutine "offset2date".'
      write(*,*)' One of the specified type of units is not recognized.'
      write(*,*)'   specified:  ',unit(first:last)
      write(*,*)'   recognized: ','years'
      write(*,*)'               ','months'
      write(*,*)'               ','days'
      write(*,*)'               ','hours'
      write(*,*)'               ','minutes'
      write(*,*)'               ','seconds'
      write(*,*)'Terminates.'
      write(*,*)
      stop

 999  write(*,*)
      write(*,*)'Error in the subroutine "offset2date".'
      write(*,*)' A negative time offset has been specified:',dt
      write(*,*)' This should be an offset from an origin date, hence'
      write(*,*)'  it cannot be negative (by definition of "origin").'
      write(*,*)'Terminates.'
      write(*,*)
      stop

 1000 return
      end



c     ********************
c     General description:
c     ********************

c     The real function "unit2unit" converts an interval of one unit to
c     the same interval with a different unit, and returns the latter.
c     If at least one of the input and output unit is either 'months'
c     or 'years', "climate years" (12 30-day months) will be used.


c     ****************
c     Input variables:
c     ****************

c      unit_in        (s)  - a character string representing the unit
c                             of the time offset of the input set,
c                             implemented:
c                              'years', 'months'   'days'
c                              'hours', 'minutes', 'seconds'
c      value          (r)  - the input time offset, in 'unit_in'
c      unit_out       (s)  - a character string representing the unit
c                             of the time offset of the output set,
c                             implemented: see 'unit_in' above

c     Above:
c      s  - string           [ character*(*) ]
c      r  - real variable    [ 4 byte, aka. "float" ]


      real function unit2unit(unit_in,value,unit_out)

      implicit none

c     Input:
      real value
      character*(*) unit_in
      character*(*) unit_out

c     Local:
      integer first_in,last_in,first_out,last_out,length

c     Function:
      integer str_pos,str_len


c     Set position of first and last relevant characters in
c     'unit_in', 'unit_out':

      length = str_len(unit_in)
      first_in = 1
      do while (unit_in(first_in:first_in) .eq. ' ')
         first_in = first_in + 1
      end do
      last_in = str_pos(unit_in,first_in,' ') - 1
      if (last_in .lt. first_in) last_in = length

      length = str_len(unit_out)
      first_out = 1
      do while (unit_out(first_out:first_out) .eq. ' ')
         first_out = first_out + 1
      end do
      last_out = str_pos(unit_out,first_out,' ') - 1
      if (last_out .lt. first_out) last_out = length

      if (unit_in(first_in:last_in) .eq. 'years') then
         if (unit_out(first_out:last_out) .eq. 'years') then
            unit2unit = value
         else if (unit_out(first_out:last_out) .eq. 'months') then
            unit2unit = value*12.
         else if (unit_out(first_out:last_out) .eq. 'days') then
            unit2unit = value*12.*30.
         else if (unit_out(first_out:last_out) .eq. 'hours') then
            unit2unit = value*12.*30.*24.
         else if (unit_out(first_out:last_out) .eq. 'minutes') then
            unit2unit = value*12.*30.*24.*60.
         else if (unit_out(first_out:last_out) .eq. 'seconds') then
            unit2unit = value*12.*30.*24.*60.*60.
         else
            go to 999
         end if
      else if (unit_in(first_in:last_in) .eq. 'months') then
         if (unit_out(first_out:last_out) .eq. 'years') then
            unit2unit = value/12.
         else if (unit_out(first_out:last_out) .eq. 'months') then
            unit2unit = value
         else if (unit_out(first_out:last_out) .eq. 'days') then
            unit2unit = value*30.
         else if (unit_out(first_out:last_out) .eq. 'hours') then
            unit2unit = value*30.*24.
         else if (unit_out(first_out:last_out) .eq. 'minutes') then
            unit2unit = value*30.*24.*60.
         else if (unit_out(first_out:last_out) .eq. 'seconds') then
            unit2unit = value*30.*24.*60.*60.
         else
            go to 999
         end if
      else if (unit_in(first_in:last_in) .eq. 'days') then
         if (unit_out(first_out:last_out) .eq. 'years') then
            unit2unit = value/(12.*30.)
         else if (unit_out(first_out:last_out) .eq. 'months') then
            unit2unit = value/30.
         else if (unit_out(first_out:last_out) .eq. 'days') then
            unit2unit = value
         else if (unit_out(first_out:last_out) .eq. 'hours') then
            unit2unit = value*24.
         else if (unit_out(first_out:last_out) .eq. 'minutes') then
            unit2unit = value*24.*60.
         else if (unit_out(first_out:last_out) .eq. 'seconds') then
            unit2unit = value*24.*60.*60.
         else
            go to 999
         end if
      else if (unit_in(first_in:last_in) .eq. 'hours') then
         if (unit_out(first_out:last_out) .eq. 'years') then
            unit2unit = value/(12.*30.*24.)
         else if (unit_out(first_out:last_out) .eq. 'months') then
            unit2unit = value/(30.*24.)
         else if (unit_out(first_out:last_out) .eq. 'days') then
            unit2unit = value/24.
         else if (unit_out(first_out:last_out) .eq. 'hours') then
            unit2unit = value
         else if (unit_out(first_out:last_out) .eq. 'minutes') then
            unit2unit = value*60.
         else if (unit_out(first_out:last_out) .eq. 'seconds') then
            unit2unit = value*60.*60.
         else
            go to 999
         end if
      else if (unit_in(first_in:last_in) .eq. 'minutes') then
         if (unit_out(first_out:last_out) .eq. 'years') then
            unit2unit = value/(12.*30.*24.*60.)
         else if (unit_out(first_out:last_out) .eq. 'months') then
            unit2unit = value/(30.*24.*60.)
         else if (unit_out(first_out:last_out) .eq. 'days') then
            unit2unit = value/(24.*60.)
         else if (unit_out(first_out:last_out) .eq. 'hours') then
            unit2unit = value/60.
         else if (unit_out(first_out:last_out) .eq. 'minutes') then
            unit2unit = value
         else if (unit_out(first_out:last_out) .eq. 'seconds') then
            unit2unit = value*60.
         else
            go to 999
         end if
      else if (unit_in(first_in:last_in) .eq. 'seconds') then
         if (unit_out(first_out:last_out) .eq. 'years') then
            unit2unit = value/(12.*30.*24.*60.*60.)
         else if (unit_out(first_out:last_out) .eq. 'months') then
            unit2unit = value/(30.*24.*60.*60.)
         else if (unit_out(first_out:last_out) .eq. 'days') then
            unit2unit = value/(24.*60.*60.)
         else if (unit_out(first_out:last_out) .eq. 'hours') then
            unit2unit = value/(60.*60.)
         else if (unit_out(first_out:last_out) .eq. 'minutes') then
            unit2unit = value*60.
         else if (unit_out(first_out:last_out) .eq. 'seconds') then
            unit2unit = value
         else
            go to 999
         end if
      else
         go to 999
      end if

      go to 1000

 999  write(*,*)
      write(*,*)'Error in the subroutine "unit2unit".'
      write(*,*)' The specified type of units is not recognized.'
      write(*,*)'   specified/input:  ',unit_in
      write(*,*)'   specified/output: ',unit_out
      write(*,*)'   recognized:       ','years'
      write(*,*)'                     ','months'
      write(*,*)'                     ','days'
      write(*,*)'                     ','hours'
      write(*,*)'                     ','minutes'
      write(*,*)'                     ','seconds'
      write(*,*)'Terminates.'
      write(*,*)
      stop

 1000 return
      end



c     ********************
c     General description:
c     ********************

c     The integer functions "m_days", "n_days" returns the number of
c     days in the specified month or year, respectively. The functions
c     recognize the use of "climate years" (12 30-day months) and
c     calendar years, as requested by the user.


c     ***************
c     Inpu variables:
c     ***************

c      climate        (l)  - switch, if (.true.) use "climate years"
c                             (12 months with 30 days in each month);
c                             if (.false.) use calendar years;
c                             disregarded if 'unit' is 'years' or
c                             'months' (then, the use of "climate
c                             years" is forced)
c      month          (i)  - the month no. to find the number of days
c                             in

c     Above:
c      l  - logical          [ (.false.)/0 or (.true.)/nonzero ]
c      i  - integer variable [ 4 byte, aka. "long" ]


      integer function m_days(climate,month,year)

      implicit none

c     Input:
      integer month,year
      logical climate

c     Local:
      integer mdays(12)
      data mdays/31,28,31,30,31,30,31,31,30,31,30,31/


c     Check 'month':
      if (month .lt. 1 .or. month .gt. 12) go to 999

      if (climate) then
         m_days = 30
      else
         m_days = mdays(month)
        if (month .eq. 2) then
          if (mod(year,4) .eq. 0) m_days = 29
          if (mod(year,100) .eq. 0 .and. mod(year,400) .ne. 0)
     >       m_days = 28
        end if
      end if

      go to 1000

 999  write(*,*)
      write(*,*)'Error in the function "m_days".'
      write(*,*)' Illegal input month no.: ',month
      write(*,*)'  Legal range: [1,12]'
      write(*,*)'Terminates.'
      write(*,*)
      stop

 1000 end


      integer function n_days(climate,year)

      implicit none

c     Input:
      integer month,year
      logical climate

      if (climate) then
         n_days = 360
      else
        n_days = 365
        if (year/4*4 .eq. year) n_days = 366
        if (year/100*100 .eq. year .and. year/400*400 .ne. year)
     >     n_days = 365
      end if

      end
